BASE_URL_ORIGINAL_MAINNET = "https://mainnet.umi.top"  # mainnet for legacy rod
BASE_URL_LEGEND_MAINNET = "https://legend.umi.top"  # mainnet for umi legend

BASE_URL_ORIGINAL_TESTNET = "https://testnet.umi.top"  # testnet for legacy rod
BASE_URL_LEGEND_TESTNET = "https://testnet-legend.umi.top"  # testnet for umi legend


CHARSET = "qpzry9x8gf2tvdw0s3jn54khce6mua7l"
BECH32M_CONST = 0x2BC830A3


MAX_TRANSACTIONS_REQUEST_LIMIT = 65535
